@@ -1,5 +1,6 @@
 from abc import ABC, abstractmethod
 from functools import wraps
+import inspect
 import os
 from typing import (
     Any,
@@ -10,10 +11,10 @@ from typing import (
     Literal,
     Optional,
     Tuple,
+    Type,
     Union,
 )
 
-from aiohttp import ClientSession
 import openai
 import pydantic
 
@@ -135,12 +136,7 @@ class GeneratorAlreadyExhaustedError(StopAsyncIteration):
     ...
 
 
-# Types
 Context = Dict[str, Any]
-TurboGen = AsyncGenerator[Union[Assistant, GetUserInput], Any]
-TurboGenFn = Callable[[Context], TurboGen]
-TurboGenTemplate = AsyncGenerator[PrefixMessage, Any]
-TurboGenTemplateFn = Callable[[Context], TurboGenTemplate]
 
 
 # Abstract classes
@@ -175,6 +171,16 @@ class BaseMemory(BasePrefixMessageCollection):
             await self.append(item)
 
 
+# Types
+TurboGen = AsyncGenerator[Union[Assistant, GetUserInput], Any]
+TurboGenFn = Callable[[Context], TurboGen]
+TurboGenTemplate = AsyncGenerator[PrefixMessage, Any]
+TurboGenTemplateFn = Union[
+    Callable[[Context], TurboGenTemplate],
+    Callable[[Context, BaseMemory], TurboGenTemplate],
+]
+
+
 # Abstract implementations
 class Example(pydantic.BaseModel, BasePrefixMessageCollection):
     user: str
@@ -226,9 +232,20 @@ def create_retry_decorator(
     )
 
 
+# Validate args for generator fn
+def validate_args(gen_fn: Callable[..., Any]) -> List[str]:
+    """Checks if function needs a memory"""
+
+    signature = inspect.signature(gen_fn)
+    arg_names = [k for k in signature.parameters.keys()]
+    assert 1 <= len(arg_names) <= 2, "Only either 1 or 2 args allowed"
+
+    return arg_names
+
+
 # Decorator
 def turbo(
-    memory: Optional[BaseMemory] = None,
+    memory_class: Type[BaseMemory] = ListMemory,
     model: TurboModel = "gpt-3.5-turbo",
     stream: bool = False,
     **kwargs,
@@ -242,10 +259,6 @@ def turbo(
         "stream": stream,
     }
 
-    # Get or create memory
-    if memory is None:
-        memory = ListMemory()
-
     # Create tenacity retry decorator
     with_retries = create_retry_decorator()
 
@@ -285,18 +298,24 @@ def turbo(
         async def turbo_gen_fn(context: Context) -> TurboGen:
             """Wrapped chatml app from an async generator"""
 
-            # Init
+            # Init memory
+            memory = memory_class()
             await memory.init(context)
-            turbo_gen = gen_fn(context)
+
+            # Init generator
+            arg_list = validate_args(gen_fn)
+            args = [context]
+
+            if len(arg_list) == 2:
+                args.append(memory)
+
+            turbo_gen = gen_fn(*args)
 
             # Parameters
             payload: Any = None
             already_yielded: bool = False
 
             try:
-                # Create a new aiohttp session for this
-                openai.aiosession.set(ClientSession())
-
                 while True:
                     # Step through the wrapped generator
                     output = await turbo_gen.asend(payload)
@@ -333,11 +352,6 @@ def turbo(
                     payload = await run_chat(memory)
                     yield payload
 
-            finally:
-                # close the http session
-                session = openai.aiosession.get()
-                session and (await session.close())
-
         return turbo_gen_fn
 
     return wrap_turbo_gen_fn
